;; eeprom_unio
;;
;; Copyright 2013 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike to mikekohn.net
;;
;; Read/write data from a 11AA160-I/TO chip using UNI/O protocol.
;; This chip is 16kbit.. 100kHz
;;

.include "msp430x2xx.inc"

RAM equ 0x0200
ERROR_FLAG equ 0x200
COMMAND equ 0x208
DST_ADDRESS equ 0x20c
SRC_ADDRESS equ 0x20e
TEXT_AREA equ 0x210

;  r4 =
;  r5 =
;  r6 =
;  r7 =
;  r8 =
;  r9 =
; r10 =
; r11 =
; r12 =
; r13 =
; r14 =
; r15 = interrupt counter

; 16MHz / 100kHz =  160 cycles
; 16MHz / 10kHz  = 1600 cycles
; Let's do 800 cycles per bit just to be safe.

PULSE_FULL equ 800 / 100
PULSE_HALF equ 400 / 100
PULSE_QUARTER equ 200 / 100

.org 0xf800
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

.if 0
  ;; Set MCLK to 16 MHz external crystal
  bic.w #OSCOFF, SR
  bis.b #XTS, &BCSCTL1
  mov.b #LFXT1S_3, &BCSCTL3
  ;mov.b #LFXT1S_3|XCAP_1, &BCSCTL3
test_osc:
  bic.b #OFIFG, &IFG1
  mov.w #0x00ff, r15
dec_again:
  dec r15
  jnz dec_again
  bit.b #(OFIFG), &IFG1
  jnz test_osc
  mov.b #(SELM_3|SELS), &BCSCTL2
.endif

  ;; Set up output pins
  ;; P1.1 = SCIO
  mov.b #0x07, &P1DIR
  mov.b #0x01, &P1OUT

  ;; Set up Timer
  mov.w #100, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  ;; Okay, I can be interrupted now
  eint

  mov.b #0, &ERROR_FLAG
  mov.b #0, &ERROR_FLAG+1
  mov.b #0, &ERROR_FLAG+2

  mov.w #TEXT_AREA, r8
memset:
  mov.b #0x55, 0(r8)
  inc.w r8
  cmp.w #TEXT_AREA+32, r8
  jne memset

  ;; it seems a prog after running once and failing fixes an issue..
  ;; but that makes no sense.
  ;call #pause_write   ; does this need time to settle?

  mov.w #text_data, &SRC_ADDRESS
  mov.w #0, &DST_ADDRESS
  call #write_data
  ;call #read_data

main:
  jmp main

write_enable:
  call #pause_standby
  call #send_header
  cmp.b #1, r5
  jne write_enable_error_1
  mov.b #0x96, r4     ; write_enable
  call #send_byte
  call #send_zero     ; MAK(0)
  call #read_bit      ; read SAK
  bis.b #0x01, &P1OUT
  cmp.b #0x01, r5
  jne write_enable_error
  ret
write_enable_error:
  mov.b #8, &ERROR_FLAG+1
  ret
write_enable_error_1:
  mov.b #7, &ERROR_FLAG
  ret

write_data:
  call #write_enable
  cmp.b #1, r5
  jne write_data_error_4
  call #pause_standby
  mov.b #16, r9                     ; max 16 bytes
  call #send_header
  cmp.b #1, r5
  jne write_data_error_1
  mov.b #0x6c, &COMMAND             ; write
  mov.b &DST_ADDRESS+1, &COMMAND+1  ; address MSB
  mov.b &DST_ADDRESS, &COMMAND+2    ; address LSB
  call #send_command
  cmp.b #1, r5
  jne write_data_error_3
  mov.w &SRC_ADDRESS, r6
  add.w #16, &DST_ADDRESS
  add.w #16, &SRC_ADDRESS
  ;mov.w #TEXT_AREA+0x10, r8    ;; DEBUG
send_next_char:
  mov.b @r6+, r7
  ;mov.b r7, 0(r8)    ;; DEBUG
  ;inc.w r8           ;; DEBUG
  mov.b r7, r4
  call #send_byte
  cmp.b #0, r7
  jeq write_data_done
  dec.b r9
  jz write_data_done
  call #send_one   ; MAK(1)
  call #read_bit   ; read SAK
  cmp.b #1, r5
  jne write_data_error 
  jmp send_next_char
write_data_done:
  call #send_zero  ; MAK(0)
  call #read_bit   ; read SAK
  bis.b #0x01, &P1OUT
  cmp.b #1, r5
  jne write_data_error_2
  ;bis.b #0x02, &P1OUT  ;; DEBUG
  call #pause_write
  tst.b r9
  jnz write_data_exit
  jmp write_data
write_data_exit:
  ret
write_data_error:
  mov.b #2, &ERROR_FLAG+1
  ret
write_data_error_1:
  mov.b #12, &ERROR_FLAG+1
  ret
write_data_error_2:
  mov.b #13, &ERROR_FLAG+1
  ret
write_data_error_3:
  mov.b #14, &ERROR_FLAG
  ret
write_data_error_4:
  mov.b #15, &ERROR_FLAG
  ret

read_data:
  call #pause_standby
  call #send_header
  cmp.b #1, r5
  jne read_data_error_1
  mov.b #0x03, &COMMAND      ; read
  mov.b #0x00, &COMMAND+1    ; address MSB
  mov.b #0x00, &COMMAND+2    ; address LSB
  call #send_command
  cmp.b #1, r5
  jnz read_data_error_2
  mov.w #TEXT_AREA, r6
read_next_char:
  ;mov.b r7, r4
  call #read_byte
  mov.b r7, 0(r6)
  cmp.b #0, r7
  jeq read_data_done
  inc.w r6
  call #send_one   ; MAK(1)
  call #read_bit   ; read SAK
  cmp.b #1, r5
  jne read_data_error 
  jmp read_next_char
read_data_done:
  call #send_zero  ; MAK(0)
  call #read_bit   ; read SAK
  bis.b #0x01, &P1OUT
  ret
read_data_error:
  mov.b #4, &ERROR_FLAG+1
  ret
read_data_error_1:
  mov.b #10, &ERROR_FLAG
  ret
read_data_error_2:
  mov.b #11, &ERROR_FLAG+1
  ret

send_header:
  call #wait_thdr
  mov.b #0x55, r4  ; send 01010101
  call #send_byte
  call #send_one   ; MAK(1)
  call #read_bit   ; read NoSAK
  mov.b #0xa0, r4  ; send address byte (10100000 or 10100000 for 11xxx1 part)
  call #send_byte
  call #send_one   ; MAK(1)
  call #read_bit   ; read SAK
  ret

send_command:
  mov.w #COMMAND, r6
send_next_command_byte:
  mov.b @r6+, r4
  call #send_byte
  call #send_one   ; MAK(1)
  call #read_bit   ; read SAK
  cmp.b #1, r5
  jne send_command_error
  cmp.w #COMMAND+3, r6
  jne send_next_command_byte
  ret
send_command_error:
  mov.b #5, &ERROR_FLAG+2
  ret

send_byte:
  mov.b #8, r5
send_bit:
  bit.b #0x80, r4
  jz next_bit_zero
  call #send_one
  jmp done_send_one
next_bit_zero:
  call #send_zero
done_send_one:
  rla.b r4
  dec.b r5
  jnz send_bit
  ret

send_zero:
  call #wait_on
  call #wait_off
  ret

send_one:
  call #wait_off
  call #wait_on
  ret

wait_on:
  bis.b #1, &P1OUT
  mov.w #0, r15
wait_on_loop:
  cmp.w #PULSE_HALF, r15
  jlo wait_on_loop
  ret

wait_off:
  bic.b #1, &P1OUT
  mov.w #0, r15
wait_off_loop:
  cmp.w #PULSE_HALF, r15
  jlo wait_off_loop
  ret

wait_thdr:
  bic.b #1, &P1OUT
  mov.w #0, r15
wait_thdr_loop:
  cmp.w #PULSE_FULL, r15
  jlo wait_thdr_loop
  bis.b #1, &P1OUT
  ret

;; read_bit() return 0, 1, or 2 for NOSAK
read_bit:
  bic.b #1, &P1DIR
  mov.w #0, r15
read_bit_1:
  cmp.w #PULSE_QUARTER, r15  ; wait 200 interrupts and then sample input pin
  jlo read_bit_1
  mov.b &P1IN, r4
  mov.w #0, r15
read_bit_2:
  cmp.w #PULSE_HALF, r15     ; wait 400 interrupts and then sample input pin
  jlo read_bit_2
  mov.b &P1IN, r5
  mov.w #0, r15
read_bit_3:
  cmp.w #PULSE_QUARTER, r15  ; wait 200 interrupts for bit to be finished
  jlo read_bit_3
  and.b #1, r4
  and.b #1, r5
  cmp.b r4, r5
  jeq return_read_error
  ;mov.b #2, r5
  bis.b #1, &P1DIR
  ret
return_read_error:
  mov.b #2, r5
  bis.b #1, &P1DIR
  ret

read_byte:
  mov.b #8, r8
  mov.b #0, r7
read_byte_next_bit:
  rla.b r7
  call #read_bit
  cmp.b #2, r5
  jeq return_read_byte_error
  bis.b r5, r7
  dec.b r8
  jnz read_byte_next_bit
  ret
return_read_byte_error:
  mov.b #9, &ERROR_FLAG
  ret

;; Stand-by pulse must be a minimum of 600us (around 9600 cycles at 16MHz)
pause_standby:
  mov.w #0, r15
pause_standby_loop:
  cmp.w #100, r15
  jlo pause_standby_loop
  ret

;; Writes take a maximum of 10ms.
pause_write:
  mov.w #0, r15
pause_write_loop:
  cmp.w #800, r15
  jlo pause_write_loop
  ret

timer_interrupt:
  inc.w r15
  reti

text_data:
  db "Michael Kohn 2013 - http://www.mikekohn.net/",0
  ;db "12345678abcdefgABCDEFGHIJKLMNOP",0
  ;db "Just testing",0

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset



